﻿if (select(2, UnitClass("player"))) ~= "ROGUE" then return end

--[[ Addon/Library Declaration ]] --
Envenomous = LibStub("AceAddon-3.0"):NewAddon("Envenomous", "AceEvent-3.0", "AceConsole-3.0", "LibBars-1.0")
local addon = Envenomous
local media = LibStub("LibSharedMedia-3.0")
local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")

--[[ Localized Globals ]]--
local _G = getfenv(0)
local UnitGUID = _G.UnitGUID
local UnitExists = _G.UnitExists
local GetSpellInfo = _G.GetSpellInfo
local GetTime = _G.GetTime
local pairs = _G.pairs

--[[ Constants ]]--
local _,_,dpicon = GetSpellInfo(57970)
local _,_,envicon = GetSpellInfo(57993)
local pName = UnitName("player")

local poisons = {
	[26968] = true,
	[27187] = true,
	[57969] = true,
	[57970] = true,
}

--[[ Variables ]]--
local tracking = {}

--[[ Addon Setup Methods ]]--
function addon:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("EnvenomousDB", self.defaults, "Default")
	AceConfig:RegisterOptionsTable("Envenomous", self.options)
	
	self:RegisterChatCommand("Envenomous", function() LibStub("AceConfigDialog-3.0"):Open("Envenomous") end )
	local optFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Envenomous", "Envenomous")
	self.options.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)

	self.db.RegisterCallback(self, "OnProfileChanged", "UpdateAnchor")
	self.db.RegisterCallback(self, "OnProfileCopied", "UpdateAnchor")
	self.db.RegisterCallback(self, "OnProfileReset", "UpdateAnchor")	

	media.RegisterCallback(self, "LibSharedMedia_Registered", "UpdateAnchor")
	media.RegisterCallback(self, "LibSharedMedia_SetGlobal", "UpdateAnchor")

	self.anchor = {}
	
	self.anchor.dp = self:NewBarGroup("DeadlyPoison", nil, self.db.profile.dp.width, self.db.profile.dp.height)
	self.anchor.dp:ClearAllPoints()
	self.anchor.dp.RegisterCallback(self, "AnchorClicked")
	self.anchor.dp:HideAnchor()
	--self:UpdateAnchor("dp")

	self.anchor.env = self:NewBarGroup("Envenom", nil, self.db.profile.envenom.width, self.db.profile.envenom.height)
	self.anchor.env:ClearAllPoints()
	self.anchor.env.RegisterCallback(self, "AnchorClicked")
	self.anchor.env:HideAnchor()
	self:UpdateAnchor()
end

function addon:OnEnable()
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	self:RegisterEvent("PLAYER_TARGET_CHANGED")
end

function addon:OnDisable()
	self:UnregisterAllEvents()
end

--[[ Timer Bar Setup Methods ]]--
function addon:AnchorClicked(callback, group, button)
	if group == self.anchor.dp then
		self:SavePosition("dp")
		if button == "RightButton" then
			group:HideAnchor()
			group:Lock()
		end
	elseif group == self.anchor.env then
		self:SavePosition("env")
		if button == "RightButton" then
			group:HideAnchor()
			group:Lock()
		end
	end
end

function addon:ToggleAnchor(group)
	if group == "dp" then
		if self.anchor.dp.button:IsVisible() then
			self.anchor.dp:HideAnchor()
			self.anchor.dp:Lock()
		else
			self.anchor.dp:ShowAnchor()
			self.anchor.dp:Unlock()
		end
	elseif group == "env" then
		if self.anchor.env.button:IsVisible() then
			self.anchor.env:HideAnchor()
			self.anchor.env:Lock()
		else
			self.anchor.env:ShowAnchor()
			self.anchor.env:Unlock()
		end
	end
end

function addon:SavePosition(group)
	if group == "dp" or not group then
		local p,_,pr,px,py = self.anchor.dp:GetPoint()
		self.db.profile.dp.position.p = p
		self.db.profile.dp.position.pr = pr
		self.db.profile.dp.position.px = px
		self.db.profile.dp.position.py = py
	end
	if group == "env" or not group then
		local p,_,pr,px,py = self.anchor.env:GetPoint()
		self.db.profile.envenom.position.p = p
		self.db.profile.envenom.position.pr = pr
		self.db.profile.envenom.position.px = px
		self.db.profile.envenom.position.py = py
	end
end

function addon:UpdateAnchor() --group)
	--if not group or group == "dp" then
		self.anchor.dp:SetTexture(media:Fetch("statusbar", self.db.profile.dp.texture))
		self.anchor.dp:SetFont(media:Fetch("font", self.db.profile.dp.font), self.db.profile.dp.fontsize)
		self.anchor.dp:SetWidth(self.db.profile.dp.width)
		self.anchor.dp:SetHeight(self.db.profile.dp.height)
		self.anchor.dp:SetFlashPeriod(5)
		self.anchor.dp:ClearAllPoints()
		self.anchor.dp:SetPoint(self.db.profile.dp.position.p or "CENTER", UIParent, self.db.profile.dp.position.pr or "CENTER", self.db.profile.dp.position.px or 0, self.db.profile.dp.position.py or 0)
		self.anchor.dp:SetScale(self.db.profile.dp.scale)
		self.anchor.dp:UnsetAllColors()
		local r, g, b, a = unpack(self.db.profile.dp.color)
		self.anchor.dp:SetColorAt(1.00, r, g, b, a)
	--end
	--if not group or group == "env" then
		self.anchor.env:SetTexture(media:Fetch("statusbar", self.db.profile.envenom.texture))
		self.anchor.env:SetFont(media:Fetch("font", self.db.profile.envenom.font), self.db.profile.envenom.fontsize)
		self.anchor.env:SetWidth(self.db.profile.envenom.width)
		self.anchor.env:SetHeight(self.db.profile.envenom.height)
		self.anchor.env:SetFlashPeriod(5)
		self.anchor.env:ClearAllPoints()
		self.anchor.env:SetPoint(self.db.profile.envenom.position.p or "CENTER", UIParent, self.db.profile.envenom.position.pr or "CENTER", self.db.profile.envenom.position.px or 0, self.db.profile.envenom.position.py or 0)
		self.anchor.env:SetScale(self.db.profile.envenom.scale)
		self.anchor.env:UnsetAllColors()
		local r, g, b, a = unpack(self.db.profile.envenom.color)
		self.anchor.env:SetColorAt(1.00, r, g, b, a)
	--end
end

--[[ Bar Management Methods ]]--
function addon:TestBar(group)
	if group == "dp" then
		self:StartBar("DPTICK", 3)
	elseif group == "env" then
		self:StartBar("ENVENOM", 6)
	end
end

function addon:StartBar(id, length)
	if id == "DPTICK" and self.db.profile.dp.enabled then
		if self.db.profile.dp.showlabel then label = "Deadly Poison" else label = "" end
		self.anchor.dp:NewTimerBar(id, label, length, 3, dpicon)
	elseif id == "ENVENOM" and self.db.profile.envenom.enabled then
		if self.db.profile.envenom.showlabel then label = "Envenom" else label = "" end
		self.anchor.env:NewTimerBar(id, label, length, 6, envicon)
	end
end

--[[ Event Handlers ]]--
function addon:COMBAT_LOG_EVENT_UNFILTERED(_, _, eventType, srcGUID, srcName, _, dstGUID, dstName, _, spellID, spellName, ...)
	if eventType == "SPELL_AURA_APPLIED" then
		if poisons[spellID] == true and srcName == pName then
			tracking[dstGUID] = GetTime()
			if dstGUID == UnitGUID("target") then 
				self:StartBar("DPTICK", 3)
			end
		elseif spellName == "Envenom" and dstName == pName then
			duration = select(6, UnitBuff("player", "Envenom"))
			self:StartBar("ENVENOM", duration)
		end
	elseif eventType == "SPELL_AURA_REFRESH" and dstName == pName and spellName == "Envenom" then
			duration = select(6, UnitBuff("player", "Envenom"))
			self:StartBar("ENVENOM", duration)
	elseif eventType == "SPELL_PERIODIC_DAMAGE" and poisons[spellID] and srcName == pName then 
		tracking[dstGUID] = GetTime()
		if (dstGUID == UnitGUID("target")) then
			self.anchor.dp:RemoveBar("DPTICK")
			self:StartBar("DPTICK", 3)
		end
	elseif eventType == "SPELL_AURA_REMOVED" and poisons[spellID] and srcName == pName then
		tracking[dstGUID] = nil
		if dstGUID == UnitGUID("target") then
			self.anchor.dp:RemoveBar("DPTICK")
		end
	elseif eventType == "UNIT_DIED" and tracking[dstGUID] ~= nil then
		tracking[dstGUID] = nil
	end
end

function addon:PLAYER_TARGET_CHANGED()
	self.anchor.dp:RemoveBar("DPTICK")
	if UnitExists("target") then
		guid = UnitGUID("target")
		if tracking[guid] then
			length = ((GetTime() - tracking[guid]) % 3.0) --find time since last tick decimal precision
			self:StartBar("DPTICK", length)
		end
	end
end
